/*
 * @(#)Vector3D.java    Pzt 10 May 2010 14:32:03 EEST
 *
 * Vector 3d for 3d world
 */
/*
*** This program wrote under GNU/General Public License ***

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.
*/
package net.dilmerkezi.point;

/**
 * @author Murat iNAN, dilmerkezi.net/eng.html
 */
public     class Vector3D
         extends Object
      implements Point3D
{

    /** x coordinate*/
    private double x;
    
    /** y coordinate*/
    private double y;
    
    /** z coordinate*/
    private double z;
    
    /** First constructor*/
    public Vector3D ()
    {
        this.x = 0.0;
        this.y = 0.0;
        this.z = 0.0;
    }
    
    /** 
     * Second constructor
     *
     * @param x x coordinate
     * @param y y coordinate
     * @param z z coordinate
     */
    public Vector3D (double xx, double yy, double zz)
    {
        this.x = xx;
        this.y = yy;
        this.z = zz;
    }
    
    /**
     * @param p Other point for subtracting than main point
     * @return Subtracted 3D point
     */
    public Point3D subtract (Point3D p)
    {
        double c1 = x-p.getX ();
        double c2 = y-p.getY ();
        double c3 = z-p.getZ ();
        return new Vector3D (c1, c2, c3);
    }

    /** @return Normalized 3D point*/
    public Point3D normalize ()
    {
        double len = this.length ();
//        double c1 = x/len;
//        double c2 = y/len;
//        double c3 = z/len;
//        return new Vector3D (c1, c2, c3);
        double norm = 1.0/len;
        Point3D p = new Vector3D (norm*x, norm*y, norm*z);
        return p;
    }
    
    /** 
     * @param p 3D point for using in reflection
     *
     * @return Reflected 3D point
     */
    public Point3D reflect (Point3D p)
    {
        Point3D pnormal = p.normalize ();
        Point3D thnormal = this.normalize ();
        double thdot = thnormal.dot (pnormal);
        Point3D pmulti = thnormal.mult (2*thdot);
        Point3D retp = pmulti.subtract (pnormal);
        return retp;
    }
    
    /**
     * @param p 3D Point for using in transformation
     * @return Transformed 3D point
     */
    public Point3D transform (Point3D p)
    {
        double c1 = x+p.getX ();
        double c2 = y+p.getY ();
        double c3 = z+p.getZ ();
        return new Vector3D (c1, c2, c3);
    }
    
    /**
     * @param p 3D Point for using in crossing
     * @return Crossed 3D point
     */
    public Point3D cross (Point3D p)
    {
        double c1 = (y*p.getZ ())-(z*p.getY ());
        double c2 = (z*p.getX ())-(x*p.getZ ());
        double c3 = (x*p.getY ())-(y*p.getX ());
        return new Vector3D (c1, c2, c3);
    }
    
    /**
     * @param d Multiplation number
     * @return Multipled 3D point
     */
    public Point3D mult (double d)
    {
        double c1 = x*d;
        double c2 = y*d;
        double c3 = z*d;
        return new Vector3D (c1, c2, c3);
    }
    
    /**
     * @param p 3D point for using in dot
     * @param Dot value
     */
    public double dot (Point3D p)
    {
        double dotp = ((x*p.getX ())+
                       (y*p.getY ())+
                       (z*p.getZ ()));
        return dotp;
    }
    
    /** @return X coordinate*/
    public double getX ()
    {
        return x;
    }
    
    /** @return Y coordinate*/
    public double getY ()
    {
        return y;
    }
    
    /** @return Z coordinate*/
    public double getZ ()
    {
        return z;
    }
    
    /** @return Length of Point3D*/
    public double length ()
    {
        double val = ((x*x)+(y*y)+(z*z));
        double sqrt = Math.sqrt (val);
        return sqrt;
    }
    
    /**
     * @param p Comparation 3D point
     * @return If equals true, otherwise false
     */
    public boolean equals (Point3D p)
    {
        return (p.getX () == x &&
                p.getY () == y &&
                p.getZ () == z);
    }
    
    /** @return Hashcode*/
    public int hashCode ()
    {
        String s = toString ();
        int result = 0;
        char [] ch = s.toCharArray ();
        int chlen = ch.length;
        int ff = 0xff;
        for (int i=0; i<chlen; i++)
        {
            result += ( ((int)(ch [i]))*ff );
        }
        return result;
    }
    
    /** @return description*/
    public String toString ()
    {  
        StringBuffer s = new StringBuffer ();
        s.append ("\nx: ");
        s.append (Double.toString (x));
        s.append ("; y: ");
        s.append (Double.toString (y));
        s.append ("; z: ");
        s.append (Double.toString (z));
        s.append ("\n");
        
        return s.toString ();
    }
    
}
/****************/
